module Fedi.Routes.Inbox where

import Prelude hiding (error)
import Fedi.Types
import Fedi.UserDetails
import Web.Twain qualified as Twain
import Web.Twain.Types qualified as Twain
import Control.Monad.IO.Class (liftIO)
import Control.Monad.Catch (catch, displayException, SomeException)
import Fedi.Signature.Check

-- * Inbox

matchInbox :: UserDetails -> Twain.PathPattern
matchInbox details =
  fromString ("/" <> details.username <> "/inbox")

handleInbox :: (AnyActivity -> Twain.ResponderM Twain.Response) -> Twain.ResponderM b
handleInbox handle = do
  let
    handler = do
      activity <- checkSignatureAndParseBody
      response <- handle activity
      Twain.send response
  handler `catch` \(e :: SomeException) -> do
    liftIO $ putStrLn (displayException e)
    Twain.send $
      Twain.status Twain.status500 $
        Twain.text "Internal Server Error 500"
